@@ -1314,6 +1314,16 @@ class sqlsrv_native_moodle_database extends moodle_database {
         }
     }
 
+    /**
+     * Does this driver support tool_replace?
+     *
+     * @since 2.6.1
+     * @return bool
+     */
+    public function replace_all_text_supported() {
+        return true;
+    }
+
     public function session_lock_supported() {
         return true;
     }
