@@ -1627,7 +1627,7 @@ function core_tables_exist() {
 function upgrade_plugin_mnet_functions($component) {
     global $DB, $CFG;
 
-    list($type, $plugin) = explode('_', $component);
+    list($type, $plugin) = core_component::normalize_component($component);
     $path = core_component::get_plugin_directory($type, $plugin);
 
     $publishes = array();
