@@ -17,8 +17,7 @@
 /**
  * Code for exporting questions as Moodle XML.
  *
- * @package    qformat
- * @subpackage xml
+ * @package    qformat_xml
  * @copyright  1999 onwards Martin Dougiamas {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -59,7 +58,7 @@ class qformat_xml extends qformat_default {
         return 'application/xml';
     }
 
-    // IMPORT FUNCTIONS START HERE
+    // IMPORT FUNCTIONS START HERE.
 
     /**
      * Translate human readable format name
@@ -107,7 +106,7 @@ class qformat_xml extends qformat_default {
      * @return string processed text.
      */
     public function import_text($text) {
-        // quick sanity check
+        // Quick sanity check.
         if (empty($text)) {
             return '';
         }
@@ -170,13 +169,11 @@ class qformat_xml extends qformat_default {
         }
         $fs = get_file_storage();
         $itemid = file_get_unused_draft_itemid();
-        $filepaths = array();
+        $filenames = array();
         foreach ($xml as $file) {
-            $filename = $this->getpath($file, array('@', 'name'), '', true);
-            $filepath = $this->getpath($file, array('@', 'path'), '/', true);
-            $fullpath = $filepath . $filename;
-            if (in_array($fullpath, $filepaths)) {
-                debugging('Duplicate file in XML: ' . $fullpath, DEBUG_DEVELOPER);
+            $filename = $file['@']['name'];
+            if (in_array($filename, $filenames)) {
+                debugging('Duplicate file in XML: ' . $filename, DEBUG_DEVELOPER);
                 continue;
             }
             $filerecord = array(
@@ -184,11 +181,11 @@ class qformat_xml extends qformat_default {
                 'component' => 'user',
                 'filearea'  => 'draft',
                 'itemid'    => $itemid,
-                'filepath'  => $filepath,
+                'filepath'  => '/',
                 'filename'  => $filename,
             );
             $fs->create_file_from_string($filerecord, base64_decode($file['#']));
-            $filepaths[] = $fullpath;
+            $filenames[] = $filename;
         }
         return $itemid;
     }
@@ -201,14 +198,10 @@ class qformat_xml extends qformat_default {
     public function import_headers($question) {
         global $CFG, $USER;
 
-        // get some error strings
-        $error_noname = get_string('xmlimportnoname', 'qformat_xml');
-        $error_noquestion = get_string('xmlimportnoquestion', 'qformat_xml');
-
-        // this routine initialises the question object
+        // This routine initialises the question object.
         $qo = $this->defaultquestion();
 
-        // Question name
+        // Question name.
         $qo->name = $this->clean_question_name($this->getpath($question,
                 array('#', 'name', 0, '#', 'text', 0, '#'), '', true,
                 get_string('xmlimportnoname', 'qformat_xml')));
@@ -240,7 +233,7 @@ class qformat_xml extends qformat_default {
             $qo->questiontext .= ' <img src="@@PLUGINFILE@@/' . $filename . '" />';
         }
 
-        // restore files in generalfeedback
+        // Restore files in generalfeedback.
         $generalfeedback = $this->import_text_with_files($question,
                 array('#', 'generalfeedback', 0), $qo->generalfeedback, $this->get_format($qo->questiontextformat));
         $qo->generalfeedback = $generalfeedback['text'];
@@ -254,7 +247,7 @@ class qformat_xml extends qformat_default {
         $qo->penalty = $this->getpath($question,
                 array('#', 'penalty', 0, '#'), $qo->penalty);
 
-        // Fix problematic rounding from old files:
+        // Fix problematic rounding from old files.
         if (abs($qo->penalty - 0.3333333) < 0.005) {
             $qo->penalty = 0.3333333;
         }
@@ -315,7 +308,7 @@ class qformat_xml extends qformat_default {
         if ($withshownumpartscorrect) {
             $qo->shownumcorrect = array_key_exists('shownumcorrect', $questionxml['#']);
 
-            // Backwards compatibility:
+            // Backwards compatibility.
             if (array_key_exists('correctresponsesfeedback', $questionxml['#'])) {
                 $qo->shownumcorrect = $this->trans_single($this->getpath($questionxml,
                         array('#', 'correctresponsesfeedback', 0, '#'), 1));
@@ -332,7 +325,7 @@ class qformat_xml extends qformat_default {
     public function import_hint($hintxml, $defaultformat) {
         $hint = new stdClass();
         if (array_key_exists('hintcontent', $hintxml['#'])) {
-            // Backwards compatibility:
+            // Backwards compatibility.
 
             $hint->hint = $this->import_text_with_files($hintxml,
                     array('#', 'hintcontent', 0), '', $defaultformat);
@@ -408,10 +401,10 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_multichoice($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // 'header' parts particular to multichoice
+        // Header parts particular to multichoice.
         $qo->qtype = 'multichoice';
         $single = $this->getpath($question, array('#', 'single', 0, '#'), 'true');
         $qo->single = $this->trans_single($single);
@@ -427,7 +420,7 @@ class qformat_xml extends qformat_default {
             $qo->answernumbering = 'abc';
         }
 
-        // Run through the answers
+        // Run through the answers.
         $answers = $question['#']['answer'];
         $acount = 0;
         foreach ($answers as $answer) {
@@ -457,7 +450,7 @@ class qformat_xml extends qformat_default {
                 array('#', 'questiontext', 0));
         $qo = qtype_multianswer_extract_question($questiontext);
 
-        // 'header' parts particular to multianswer
+        // Header parts particular to multianswer.
         $qo->qtype = 'multianswer';
         $qo->course = $this->course;
 
@@ -489,7 +482,7 @@ class qformat_xml extends qformat_default {
             $qo->questiontext .= ' <img src="@@PLUGINFILE@@/' . $filename . '" />';
         }
 
-        // restore files in generalfeedback
+        // Restore files in generalfeedback.
         $generalfeedback = $this->import_text_with_files($question,
                 array('#', 'generalfeedback', 0), $qo->generalfeedback, $this->get_format($qo->questiontextformat));
         $qo->generalfeedback = $generalfeedback['text'];
@@ -500,7 +493,7 @@ class qformat_xml extends qformat_default {
 
         $qo->penalty = $this->getpath($question,
                 array('#', 'penalty', 0, '#'), $this->defaultquestion()->penalty);
-        // Fix problematic rounding from old files:
+        // Fix problematic rounding from old files.
         if (abs($qo->penalty - 0.3333333) < 0.005) {
             $qo->penalty = 0.3333333;
         }
@@ -516,11 +509,11 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_truefalse($question) {
-        // get common parts
+        // Get common parts.
         global $OUTPUT;
         $qo = $this->import_headers($question);
 
-        // 'header' parts particular to true/false
+        // Header parts particular to true/false.
         $qo->qtype = 'truefalse';
 
         // In the past, it used to be assumed that the two answers were in the file
@@ -575,16 +568,16 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_shortanswer($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // header parts particular to shortanswer
+        // Header parts particular to shortanswer.
         $qo->qtype = 'shortanswer';
 
-        // get usecase
+        // Get usecase.
         $qo->usecase = $this->getpath($question, array('#', 'usecase', 0, '#'), $qo->usecase);
 
-        // Run through the answers
+        // Run through the answers.
         $answers = $question['#']['answer'];
         $acount = 0;
         foreach ($answers as $answer) {
@@ -606,9 +599,9 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_description($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
-        // header parts particular to shortanswer
+        // Header parts particular to shortanswer.
         $qo->qtype = 'description';
         $qo->defaultmark = 0;
         $qo->length = 0;
@@ -621,20 +614,20 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_numerical($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // header parts particular to numerical
+        // Header parts particular to numerical.
         $qo->qtype = 'numerical';
 
-        // get answers array
+        // Get answers array.
         $answers = $question['#']['answer'];
         $qo->answer = array();
         $qo->feedback = array();
         $qo->fraction = array();
         $qo->tolerance = array();
         foreach ($answers as $answer) {
-            // answer outside of <text> is deprecated
+            // Answer outside of <text> is deprecated.
             $obj = $this->import_answer($answer, false, $this->get_format($qo->questiontextformat));
             $qo->answer[] = $obj->answer['text'];
             if (empty($qo->answer)) {
@@ -643,13 +636,13 @@ class qformat_xml extends qformat_default {
             $qo->feedback[]  = $obj->feedback;
             $qo->tolerance[] = $this->getpath($answer, array('#', 'tolerance', 0, '#'), 0);
 
-            // fraction as a tag is deprecated
+            // Fraction as a tag is deprecated.
             $fraction = $this->getpath($answer, array('@', 'fraction'), 0) / 100;
             $qo->fraction[] = $this->getpath($answer,
-                    array('#', 'fraction', 0, '#'), $fraction); // deprecated
+                    array('#', 'fraction', 0, '#'), $fraction); // Deprecated.
         }
 
-        // Get the units array
+        // Get the units array.
         $qo->unit = array();
         $units = $this->getpath($question, array('#', 'units', 0, '#', 'unit'), array());
         if (!empty($units)) {
@@ -674,9 +667,9 @@ class qformat_xml extends qformat_default {
         if (is_null($qo->showunits)) {
             // Set a good default, depending on whether there are any units defined.
             if (empty($qo->unit)) {
-                $qo->showunits = 3; // qtype_numerical::UNITNONE;
+                $qo->showunits = 3; // This is qtype_numerical::UNITNONE, but we cannot refer to that constant here.
             } else {
-                $qo->showunits = 0; // qtype_numerical::UNITOPTIONAL;
+                $qo->showunits = 0; // This is qtype_numerical::UNITOPTIONAL, but we cannot refer to that constant here.
             }
         }
 
@@ -691,15 +684,15 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_match($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // header parts particular to matching
+        // Header parts particular to matching.
         $qo->qtype = 'match';
         $qo->shuffleanswers = $this->trans_single($this->getpath($question,
                 array('#', 'shuffleanswers', 0, '#'), 1));
 
-        // run through subquestions
+        // Run through subquestions.
         $qo->subquestions = array();
         $qo->subanswers = array();
         foreach ($question['#']['subquestion'] as $subqxml) {
@@ -723,10 +716,10 @@ class qformat_xml extends qformat_default {
      * @return object question object
      */
     public function import_essay($question) {
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // header parts particular to essay
+        // Header parts particular to essay.
         $qo->qtype = 'essay';
 
         $qo->responseformat = $this->getpath($question,
@@ -751,10 +744,10 @@ class qformat_xml extends qformat_default {
      */
     public function import_calculated($question) {
 
-        // get common parts
+        // Get common parts.
         $qo = $this->import_headers($question);
 
-        // header parts particular to calculated
+        // Header parts particular to calculated.
         $qo->qtype = 'calculated';
         $qo->synchronize = $this->getpath($question, array('#', 'synchronize', 0, '#'), 0);
         $single = $this->getpath($question, array('#', 'single', 0, '#'), 'true');
@@ -778,9 +771,9 @@ class qformat_xml extends qformat_default {
                     array('0'), '', $this->get_format($qo->questiontextformat));
         }
 
-        // get answers array
+        // Get answers array.
         $answers = $question['#']['answer'];
-        $qo->answer = array();
+        $qo->answers = array();
         $qo->feedback = array();
         $qo->fraction = array();
         $qo->tolerance = array();
@@ -790,14 +783,14 @@ class qformat_xml extends qformat_default {
         $qo->feedback = array();
         foreach ($answers as $answer) {
             $ans = $this->import_answer($answer, true, $this->get_format($qo->questiontextformat));
-            // answer outside of <text> is deprecated
+            // Answer outside of <text> is deprecated.
             if (empty($ans->answer['text'])) {
                 $ans->answer['text'] = '*';
             }
-            $qo->answer[] = $ans->answer['text'];
+            $qo->answers[] = $ans->answer;
             $qo->feedback[] = $ans->feedback;
             $qo->tolerance[] = $answer['#']['tolerance'][0]['#'];
-            // fraction as a tag is deprecated
+            // Fraction as a tag is deprecated.
             if (!empty($answer['#']['fraction'][0]['#'])) {
                 $qo->fraction[] = $answer['#']['fraction'][0]['#'];
             } else {
@@ -807,7 +800,7 @@ class qformat_xml extends qformat_default {
             $qo->correctanswerformat[] = $answer['#']['correctanswerformat'][0]['#'];
             $qo->correctanswerlength[] = $answer['#']['correctanswerlength'][0]['#'];
         }
-        // get units array
+        // Get units array.
         $qo->unit = array();
         if (isset($question['#']['units'][0]['#']['unit'])) {
             $units = $question['#']['units'][0]['#']['unit'];
@@ -826,9 +819,9 @@ class qformat_xml extends qformat_default {
         if (is_null($qo->unitpenalty)) {
             // Set a good default, depending on whether there are any units defined.
             if (empty($qo->unit)) {
-                $qo->showunits = 3; // qtype_numerical::UNITNONE;
+                $qo->showunits = 3; // This is qtype_numerical::UNITNONE, but we cannot refer to that constant here.
             } else {
-                $qo->showunits = 0; // qtype_numerical::UNITOPTIONAL;
+                $qo->showunits = 0; // This is qtype_numerical::UNITOPTIONAL, but we cannot refer to that constant here.
             }
         }
 
@@ -901,11 +894,11 @@ class qformat_xml extends qformat_default {
      * @return array (of objects) question objects.
      */
     protected function readquestions($lines) {
-        // We just need it as one big string
+        // We just need it as one big string.
         $lines = implode('', $lines);
 
         // This converts xml to big nasty data structure
-        // the 0 means keep white space as it is (important for markdown format)
+        // the 0 means keep white space as it is (important for markdown format).
         try {
             $xml = xmlize($lines, 0, 'UTF-8', true);
         } catch (xml_format_exception $e) {
@@ -923,11 +916,11 @@ class qformat_xml extends qformat_default {
     public function import_questions($xml) {
         $questions = array();
 
-        // Iterate through questions
+        // Iterate through questions.
         foreach ($xml as $questionxml) {
             $qo = $this->import_question($questionxml);
 
-            // Stick the result in the $questions array
+            // Stick the result in the $questions array.
             if ($qo) {
                 $questions[] = $qo;
             }
@@ -982,7 +975,7 @@ class qformat_xml extends qformat_default {
         }
     }
 
-    // EXPORT FUNCTIONS START HERE
+    // EXPORT FUNCTIONS START HERE.
 
     public function export_file_extension() {
         return '.xml';
@@ -1095,15 +1088,15 @@ class qformat_xml extends qformat_default {
             if ($file->is_directory()) {
                 continue;
             }
-            $string .= '<file name="' . $file->get_filename() . '" path="' . $file->get_filepath() . '" encoding="base64">';
+            $string .= '<file name="' . $file->get_filename() . '" encoding="base64">';
             $string .= base64_encode($file->get_content());
-            $string .= "</file>\n";
+            $string .= '</file>';
         }
         return $string;
     }
 
     protected function presave_process($content) {
-        // Override to allow us to add xml headers and footers
+        // Override to allow us to add xml headers and footers.
         return '<?xml version="1.0" encoding="UTF-8"?>
 <quiz>
 ' . $content . '</quiz>';
@@ -1140,7 +1133,7 @@ class qformat_xml extends qformat_default {
         // Add a comment linking this to the original question id.
         $expout .= "<!-- question: $question->id  -->\n";
 
-        // Check question type
+        // Check question type.
         $questiontype = $this->get_qtype($question->qtype);
 
         // Categories are a special case.
@@ -1177,7 +1170,7 @@ class qformat_xml extends qformat_default {
         // The rest of the output depends on question type.
         switch($question->qtype) {
             case 'category':
-                // not a qtype really - dummy used for category switching
+                // Not a qtype really - dummy used for category switching.
                 break;
 
             case 'truefalse':
@@ -1273,7 +1266,7 @@ class qformat_xml extends qformat_default {
 
             case 'multianswer':
                 foreach ($question->options->questions as $index => $subq) {
-                    $expout = str_replace('{#' . $index . '}', $subq->questiontext, $expout);
+                    $expout = preg_replace('~{#' . $index . '}~', $subq->questiontext, $expout);
                 }
                 break;
 
@@ -1331,7 +1324,7 @@ class qformat_xml extends qformat_default {
                 foreach ($question->options->answers as $answer) {
                     $percent = 100 * $answer->fraction;
                     $expout .= "<answer fraction=\"$percent\">\n";
-                    // "<text/>" tags are an added feature, old files won't have them
+                    // The "<text/>" tags are an added feature, old files won't have them.
                     $expout .= "    <text>{$answer->answer}</text>\n";
                     $expout .= "    <tolerance>{$answer->tolerance}</tolerance>\n";
                     $expout .= "    <tolerancetype>{$answer->tolerancetype}</tolerancetype>\n";
@@ -1389,7 +1382,7 @@ class qformat_xml extends qformat_default {
                 // The tag $question->export_process has been set so we get all the
                 // data items in the database from the function
                 // qtype_calculated::get_question_options calculatedsimple defaults
-                // to calculated
+                // to calculated.
                 if (isset($question->options->datasets) && count($question->options->datasets)) {
                     $expout .= "<dataset_definitions>\n";
                     foreach ($question->options->datasets as $def) {
@@ -1453,7 +1446,7 @@ class qformat_xml extends qformat_default {
             }
         }
 
-        // close the question tag
+        // Close the question tag.
         $expout .= "  </question>\n";
         if ($invalidquestion) {
             return '';
