@@ -138,7 +138,7 @@ abstract class question_edit_form extends question_wizard_form {
                 $contexts = $this->contexts->having_cap('moodle/question:add');
             }
 
-            // Adding question
+            // Adding question.
             $mform->addElement('questioncategory', 'category', get_string('category', 'question'),
                     array('contexts' => $contexts));
         } else if (!($this->question->formoptions->canmove ||
@@ -153,14 +153,14 @@ abstract class question_edit_form extends question_wizard_form {
                     array('contexts' => $this->contexts->having_cap('moodle/question:add')));
 
         } else {
-            // Editing question with permission to move from category or save as new q
+            // Editing question with permission to move from category or save as new q.
             $currentgrp = array();
             $currentgrp[0] = $mform->createElement('questioncategory', 'category',
                     get_string('categorycurrent', 'question'),
                     array('contexts' => array($this->categorycontext)));
             if ($this->question->formoptions->canedit ||
                     $this->question->formoptions->cansaveasnew) {
-                //not move only form
+                // Not move only form.
                 $currentgrp[1] = $mform->createElement('checkbox', 'usecurrentcat', '',
                         get_string('categorycurrentuse', 'question'));
                 $mform->setDefault('usecurrentcat', 1);
@@ -175,7 +175,7 @@ abstract class question_edit_form extends question_wizard_form {
                     array('contexts' => array($this->categorycontext)));
             if ($this->question->formoptions->canedit ||
                     $this->question->formoptions->cansaveasnew) {
-                //not move only form
+                // Not move only form.
                 $mform->disabledIf('categorymoveto', 'usecurrentcat', 'checked');
             }
         }
@@ -188,6 +188,7 @@ abstract class question_edit_form extends question_wizard_form {
         $mform->addElement('editor', 'questiontext', get_string('questiontext', 'question'),
                 array('rows' => 15), $this->editoroptions);
         $mform->setType('questiontext', PARAM_RAW);
+        $mform->addRule('questiontext', null, 'required', null, 'client');
 
         $mform->addElement('text', 'defaultmark', get_string('defaultmark', 'question'),
                 array('size' => 7));
@@ -242,7 +243,7 @@ abstract class question_edit_form extends question_wizard_form {
 
         $buttonarray = array();
         if (!empty($this->question->id)) {
-            // Editing / moving question
+            // Editing / moving question.
             if ($this->question->formoptions->movecontext) {
                 $buttonarray[] = $mform->createElement('submit', 'submitbutton',
                         get_string('moveq', 'question'));
@@ -256,7 +257,7 @@ abstract class question_edit_form extends question_wizard_form {
             }
             $buttonarray[] = $mform->createElement('cancel');
         } else {
-            // Adding new question
+            // Adding new question.
             $buttonarray[] = $mform->createElement('submit', 'submitbutton',
                     get_string('savechanges'));
             $buttonarray[] = $mform->createElement('cancel');
@@ -294,11 +295,13 @@ abstract class question_edit_form extends question_wizard_form {
     protected function get_per_answer_fields($mform, $label, $gradeoptions,
             &$repeatedoptions, &$answersoption) {
         $repeated = array();
-        $repeated[] = $mform->createElement('header', 'answerhdr', $label);
-        $repeated[] = $mform->createElement('text', 'answer',
-                get_string('answer', 'question'), array('size' => 80));
-        $repeated[] = $mform->createElement('select', 'fraction',
+        $answeroptions = array();
+        $answeroptions[] = $mform->createElement('text', 'answer',
+                $label, array('size' => 40));
+        $answeroptions[] = $mform->createElement('select', 'fraction',
                 get_string('grade'), $gradeoptions);
+        $repeated[] = $mform->createElement('group', 'answeroptions',
+                 $label, $answeroptions, null, false);
         $repeated[] = $mform->createElement('editor', 'feedback',
                 get_string('feedback', 'question'), array('rows' => 5), $this->editoroptions);
         $repeatedoptions['answer']['type'] = PARAM_RAW;
@@ -319,25 +322,30 @@ abstract class question_edit_form extends question_wizard_form {
      */
     protected function add_per_answer_fields(&$mform, $label, $gradeoptions,
             $minoptions = QUESTION_NUMANS_START, $addoptions = QUESTION_NUMANS_ADD) {
+        $mform->addElement('header', 'answerhdr',
+                    get_string('answers', 'question'), '');
+        $mform->setExpanded('answerhdr', 1);
         $answersoption = '';
         $repeatedoptions = array();
         $repeated = $this->get_per_answer_fields($mform, $label, $gradeoptions,
                 $repeatedoptions, $answersoption);
 
         if (isset($this->question->options)) {
-            $countanswers = count($this->question->options->$answersoption);
+            $repeatsatstart = count($this->question->options->$answersoption);
         } else {
-            $countanswers = 0;
-        }
-        if ($this->question->formoptions->repeatelements) {
-            $repeatsatstart = max($minoptions, $countanswers + $addoptions);
-        } else {
-            $repeatsatstart = $countanswers;
+            $repeatsatstart = $minoptions;
         }
 
         $this->repeat_elements($repeated, $repeatsatstart, $repeatedoptions,
                 'noanswers', 'addanswers', $addoptions,
-                get_string('addmorechoiceblanks', 'qtype_multichoice'));
+                $this->get_more_choices_string(), true);
+    }
+
+    /**
+     * Language string to use for 'Add {no} more {whatever we call answers}'.
+     */
+    protected function get_more_choices_string() {
+        return get_string('addmorechoiceblanks', 'question');
     }
 
     protected function add_combined_feedback_fields($withshownumpartscorrect = false) {
@@ -348,36 +356,52 @@ abstract class question_edit_form extends question_wizard_form {
 
         $fields = array('correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback');
         foreach ($fields as $feedbackname) {
-            $mform->addElement('editor', $feedbackname, get_string($feedbackname, 'question'),
+            $element = $mform->addElement('editor', $feedbackname,
+                                get_string($feedbackname, 'question'),
                                 array('rows' => 5), $this->editoroptions);
             $mform->setType($feedbackname, PARAM_RAW);
+            // Using setValue() as setDefault() does not work for the editor class.
+            $element->setValue(array('text'=>get_string($feedbackname.'default', 'question')));
 
             if ($withshownumpartscorrect && $feedbackname == 'partiallycorrectfeedback') {
                 $mform->addElement('advcheckbox', 'shownumcorrect',
                         get_string('options', 'question'),
                         get_string('shownumpartscorrectwhenfinished', 'question'));
+                $mform->setDefault('shownumcorrect', true);
             }
         }
     }
 
+    /**
+     * Create the form elements required by one hint.
+     * @param string $withclearwrong whether this quesiton type uses the 'Clear wrong' option on hints.
+     * @param string $withshownumpartscorrect whether this quesiton type uses the 'Show num parts correct' option on hints.
+     * @return array form field elements for one hint.
+     */
     protected function get_hint_fields($withclearwrong = false, $withshownumpartscorrect = false) {
         $mform = $this->_form;
 
+        $repeatedoptions = array();
         $repeated = array();
-        $repeated[] = $mform->createElement('header', 'hinthdr', get_string('hintn', 'question'));
-        $repeated[] = $mform->createElement('editor', 'hint', get_string('hinttext', 'question'),
+        $repeated[] = $mform->createElement('editor', 'hint', get_string('hintn', 'question'),
                 array('rows' => 5), $this->editoroptions);
         $repeatedoptions['hint']['type'] = PARAM_RAW;
 
+        $optionelements = array();
         if ($withclearwrong) {
-            $repeated[] = $mform->createElement('advcheckbox', 'hintclearwrong',
+            $optionelements[] = $mform->createElement('advcheckbox', 'hintclearwrong',
                     get_string('options', 'question'), get_string('clearwrongparts', 'question'));
         }
         if ($withshownumpartscorrect) {
-            $repeated[] = $mform->createElement('advcheckbox', 'hintshownumcorrect', '',
+            $optionelements[] = $mform->createElement('advcheckbox', 'hintshownumcorrect', '',
                     get_string('shownumpartscorrect', 'question'));
         }
 
+        if (count($optionelements)) {
+            $repeated[] = $mform->createElement('group', 'hintoptions',
+                 get_string('hintnoptions', 'question'), $optionelements, null, false);
+        }
+
         return array($repeated, $repeatedoptions);
     }
 
@@ -407,7 +431,6 @@ abstract class question_edit_form extends question_wizard_form {
         }
         $mform->addElement('select', 'penalty',
                 get_string('penaltyforeachincorrecttry', 'question'), $penaltyoptions);
-        $mform->addRule('penalty', null, 'required', null, 'client');
         $mform->addHelpButton('penalty', 'penaltyforeachincorrecttry', 'question');
         $mform->setDefault('penalty', 0.3333333);
 
@@ -426,13 +449,13 @@ abstract class question_edit_form extends question_wizard_form {
         list($repeated, $repeatedoptions) = $this->get_hint_fields(
                 $withclearwrong, $withshownumpartscorrect);
         $this->repeat_elements($repeated, $repeatsatstart, $repeatedoptions,
-                'numhints', 'addhint', 1, get_string('addanotherhint', 'question'));
+                'numhints', 'addhint', 1, get_string('addanotherhint', 'question'), true);
     }
 
     public function set_data($question) {
         question_bank::get_qtype($question->qtype)->set_default_options($question);
 
-        // prepare question text
+        // Prepare question text.
         $draftid = file_get_submitted_draft_itemid('questiontext');
 
         if (!empty($question->questiontext)) {
@@ -451,7 +474,7 @@ abstract class question_edit_form extends question_wizard_form {
                 editors_get_preferred_format() : $question->questiontextformat;
         $question->questiontext['itemid'] = $draftid;
 
-        // prepare general feedback
+        // Prepare general feedback.
         $draftid = file_get_submitted_draft_itemid('generalfeedback');
 
         if (empty($question->generalfeedback)) {
@@ -481,13 +504,13 @@ abstract class question_edit_form extends question_wizard_form {
         if (is_array($extraquestionfields) && !empty($question->options)) {
             array_shift($extraquestionfields);
             foreach ($extraquestionfields as $field) {
-                if (isset($question->options->$field)) {
+                if (property_exists($question->options, $field)) {
                     $question->$field = $question->options->$field;
                 }
             }
         }
 
-        // subclass adds data_preprocessing code here
+        // Subclass adds data_preprocessing code here.
         $question = $this->data_preprocessing($question);
 
         parent::set_data($question);
@@ -517,16 +540,16 @@ abstract class question_edit_form extends question_wizard_form {
         $key = 0;
         foreach ($question->options->answers as $answer) {
             if ($withanswerfiles) {
-                // Prepare the feedback editor to display files in draft area
+                // Prepare the feedback editor to display files in draft area.
                 $draftitemid = file_get_submitted_draft_itemid('answer['.$key.']');
                 $question->answer[$key]['text'] = file_prepare_draft_area(
-                    $draftitemid,          // draftid
+                    $draftitemid,          // Draftid
                     $this->context->id,    // context
                     'question',            // component
                     'answer',              // filarea
                     !empty($answer->id) ? (int) $answer->id : null, // itemid
                     $this->fileoptions,    // options
-                    $answer->answer        // text
+                    $answer->answer        // text.
                 );
                 $question->answer[$key]['itemid'] = $draftitemid;
                 $question->answer[$key]['format'] = $answer->answerformat;
@@ -545,19 +568,19 @@ abstract class question_edit_form extends question_wizard_form {
             // that ->_defaultValues['fraction[0]'] has already been set, but we
             // are using object notation here, so we will be setting
             // ->_defaultValues['fraction'][0]. That does not work, so we have
-            // to unset ->_defaultValues['fraction[0]']
+            // to unset ->_defaultValues['fraction[0]'].
             unset($this->_form->_defaultValues["fraction[$key]"]);
 
-            // Prepare the feedback editor to display files in draft area
+            // Prepare the feedback editor to display files in draft area.
             $draftitemid = file_get_submitted_draft_itemid('feedback['.$key.']');
             $question->feedback[$key]['text'] = file_prepare_draft_area(
-                $draftitemid,          // draftid
+                $draftitemid,          // Draftid
                 $this->context->id,    // context
                 'question',            // component
                 'answerfeedback',      // filarea
                 !empty($answer->id) ? (int) $answer->id : null, // itemid
                 $this->fileoptions,    // options
-                $answer->feedback      // text
+                $answer->feedback      // text.
             );
             $question->feedback[$key]['itemid'] = $draftitemid;
             $question->feedback[$key]['format'] = $answer->feedbackformat;
@@ -583,13 +606,13 @@ abstract class question_edit_form extends question_wizard_form {
             $draftid = file_get_submitted_draft_itemid($feedbackname);
             $feedback = array();
             $feedback['text'] = file_prepare_draft_area(
-                $draftid,              // draftid
+                $draftid,              // Draftid
                 $this->context->id,    // context
                 'question',            // component
                 $feedbackname,         // filarea
                 !empty($question->id) ? (int) $question->id : null, // itemid
                 $this->fileoptions,    // options
-                $question->options->$feedbackname // text
+                $question->options->$feedbackname // text.
             );
             $feedbackformat = $feedbackname . 'format';
             $feedback['format'] = $question->options->$feedbackformat;
@@ -620,16 +643,16 @@ abstract class question_edit_form extends question_wizard_form {
         foreach ($question->hints as $hint) {
             $question->hint[$key] = array();
 
-            // prepare feedback editor to display files in draft area
+            // Prepare feedback editor to display files in draft area.
             $draftitemid = file_get_submitted_draft_itemid('hint['.$key.']');
             $question->hint[$key]['text'] = file_prepare_draft_area(
-                $draftitemid,          // draftid
+                $draftitemid,          // Draftid
                 $this->context->id,    // context
                 'question',            // component
                 'hint',                // filarea
                 !empty($hint->id) ? (int) $hint->id : null, // itemid
                 $this->fileoptions,    // options
-                $hint->hint            // text
+                $hint->hint            // text.
             );
             $question->hint[$key]['itemid'] = $draftitemid;
             $question->hint[$key]['format'] = $hint->hintformat;
@@ -654,6 +677,12 @@ abstract class question_edit_form extends question_wizard_form {
                 && empty($fromform['usecurrentcat']) && !$this->question->formoptions->canmove) {
             $errors['currentgrp'] = get_string('nopermissionmove', 'question');
         }
+
+        // Default mark.
+        if (array_key_exists('defaultmark', $fromform) && $fromform['defaultmark'] < 0) {
+            $errors['defaultmark'] = get_string('defaultmarkmustbepositive', 'question');
+        }
+
         return $errors;
     }
 
@@ -663,4 +692,15 @@ abstract class question_edit_form extends question_wizard_form {
      *      in the question type class.
      */
     public abstract function qtype();
+
+    /**
+     * Returns an array of editor options with collapsed options turned off.
+     * @deprecated since 2.6
+     * @return array
+     */
+    protected function get_non_collabsible_editor_options() {
+        debugging('get_non_collabsible_editor_options() is deprecated, use $this->editoroptions instead.', DEBUG_DEVELOPER);
+        return $this->editoroptions;
+    }
+
 }
